@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
  * http://www.tanukisoftware.comment
  * All rights reserved.
  *
@@ -58,6 +58,9 @@
 
 #define MAX_INCLUDE_DEPTH 10
 
+/* The largest possible "name+'='+value" property pair length on Windows. */
+#define MAX_ENV_PAIR_LEN 32767
+
 EnvSrc *baseEnvSrc = NULL;
 
 /** Stores the time that the property file began to be loaded. */
@@ -196,27 +199,30 @@ TCHAR generateValueBuffer[256];
 
 /**
  * This function returns a reference to a static buffer and is NOT thread safe.
+ *  It is currently called only when loading a property file and when firing an event.
+ *  Both happen in the main thread.
+ * The largest return value can be 15+1 characters.
  */
-TCHAR* generateTimeValue(const TCHAR* format) {
+TCHAR* generateTimeValue(const TCHAR* format, struct tm *timeTM) {
     if (strcmpIgnoreCase(format, TEXT("YYYYMMDDHHIISS")) == 0) {
         _sntprintf(generateValueBuffer, 256, TEXT("%04d%02d%02d%02d%02d%02d"),
-        loadPropertiesTM.tm_year + 1900, loadPropertiesTM.tm_mon + 1, loadPropertiesTM.tm_mday,
-        loadPropertiesTM.tm_hour, loadPropertiesTM.tm_min, loadPropertiesTM.tm_sec );
+        timeTM->tm_year + 1900, timeTM->tm_mon + 1, timeTM->tm_mday,
+        timeTM->tm_hour, timeTM->tm_min, timeTM->tm_sec );
     } else if (strcmpIgnoreCase(format, TEXT("YYYYMMDD_HHIISS")) == 0) {
         _sntprintf(generateValueBuffer, 256, TEXT("%04d%02d%02d_%02d%02d%02d"),
-        loadPropertiesTM.tm_year + 1900, loadPropertiesTM.tm_mon + 1, loadPropertiesTM.tm_mday,
-        loadPropertiesTM.tm_hour, loadPropertiesTM.tm_min, loadPropertiesTM.tm_sec );
+        timeTM->tm_year + 1900, timeTM->tm_mon + 1, timeTM->tm_mday,
+        timeTM->tm_hour, timeTM->tm_min, timeTM->tm_sec );
     } else if (strcmpIgnoreCase(format, TEXT("YYYYMMDDHHII")) == 0) {
         _sntprintf(generateValueBuffer, 256, TEXT("%04d%02d%02d%02d%02d"),
-        loadPropertiesTM.tm_year + 1900, loadPropertiesTM.tm_mon + 1, loadPropertiesTM.tm_mday,
-        loadPropertiesTM.tm_hour, loadPropertiesTM.tm_min );
+        timeTM->tm_year + 1900, timeTM->tm_mon + 1, timeTM->tm_mday,
+        timeTM->tm_hour, timeTM->tm_min );
     } else if (strcmpIgnoreCase(format, TEXT("YYYYMMDDHH")) == 0) {
         _sntprintf(generateValueBuffer, 256, TEXT("%04d%02d%02d%02d"),
-        loadPropertiesTM.tm_year + 1900, loadPropertiesTM.tm_mon + 1, loadPropertiesTM.tm_mday,
-        loadPropertiesTM.tm_hour );
+        timeTM->tm_year + 1900, timeTM->tm_mon + 1, timeTM->tm_mday,
+        timeTM->tm_hour );
     } else if (strcmpIgnoreCase(format, TEXT("YYYYMMDD")) == 0) {
         _sntprintf(generateValueBuffer, 256, TEXT("%04d%02d%02d"),
-        loadPropertiesTM.tm_year + 1900, loadPropertiesTM.tm_mon + 1, loadPropertiesTM.tm_mday);
+        timeTM->tm_year + 1900, timeTM->tm_mon + 1, timeTM->tm_mday);
     } else {
         _sntprintf(generateValueBuffer, 256, TEXT("{INVALID}"));
     }
@@ -225,6 +231,9 @@ TCHAR* generateTimeValue(const TCHAR* format) {
 
 /**
  * This function returns a reference to a static buffer and is NOT thread safe.
+ *  It is currently called only when loading a property file and when firing an event.
+ *  Both happen in the main thread.
+ * The largest return value can be 9+1 characters.
  */
 TCHAR* generateRandValue(const TCHAR* format) {
     if (strcmpIgnoreCase(format, TEXT("N")) == 0) {
@@ -300,7 +309,7 @@ void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, int
                 envValueNeedFree = FALSE;
                 if (_tcsstr(envName, TEXT("WRAPPER_TIME_")) == envName) {
                     /* Found a time value. */
-                    envValue = generateTimeValue(envName + 13);
+                    envValue = generateTimeValue(envName + 13, &loadPropertiesTM);
                 } else if (_tcsstr(envName, TEXT("WRAPPER_RAND_")) == envName) {
                     /* Found a time value. */
                     envValue = generateRandValue(envName + 13);
@@ -459,13 +468,19 @@ void setInnerProperty(Properties *properties, Property *property, const TCHAR *p
     }
 }
 
-static int loadPropertiesCallback(void *callbackParam, const TCHAR *fileName, int lineNumber, TCHAR *config, int debugProperties)
+static int loadPropertiesCallback(void *callbackParam, const TCHAR *fileName, int lineNumber, TCHAR *config, int exitOnOverwrite, int logLevelOnOverwrite)
 {
     Properties *properties = (Properties *)callbackParam;
     TCHAR *d;
 
-    properties->debugProperties = debugProperties;
+    properties->exitOnOverwrite = exitOnOverwrite;
+    properties->logLevelOnOverwrite = logLevelOnOverwrite;
 
+    /* special case where the callback should only update the properties structure */
+    if ((fileName == NULL) && (lineNumber == -1) && (config == NULL)) {
+        return TRUE;
+    }
+    
     if (_tcsstr(config, TEXT("include")) == config) {
         /* Users sometimes remove the '#' from include statements.
            Add a warning to help them notice the problem. */
@@ -489,22 +504,30 @@ static int loadPropertiesCallback(void *callbackParam, const TCHAR *fileName, in
  * @param properties Properties structure to load into.
  * @param filename File to load the properties from.
  * @param preload TRUE if this is a preload call that should have supressed error output.
+ * @param argCommand Argument passed to the binary.
+ * @param originalWorkingDir Working directory of the binary at the moment it was launched.
+ * @param isDebugging Flag that controls whether or not debug output will be logged.
  *
  * @return TRUE if there were any problems, FALSE if successful.
  */
-int loadProperties(Properties *properties, const TCHAR* filename, int preload) {
+int loadProperties(Properties *properties,
+                   const TCHAR* filename,
+                   int preload,
+                   const TCHAR *argCommand,
+                   const TCHAR *originalWorkingDir,
+                   int isDebugging) {
     /* Store the time that the property file began to be loaded. */
     #ifdef WIN32
     struct _timeb timebNow;
     #else
     struct timeval timevalNow;
     #endif
-    time_t      now;
-    struct tm   *nowTM;
+    time_t now;
+    struct tm *nowTM;
     int loadResult;
     
 #ifdef WIN32
-    _ftime( &timebNow );
+    _ftime(&timebNow);
     now = (time_t)timebNow.time;
 #else
     gettimeofday(&timevalNow, NULL);
@@ -513,7 +536,7 @@ int loadProperties(Properties *properties, const TCHAR* filename, int preload) {
     nowTM = localtime(&now);
     memcpy(&loadPropertiesTM, nowTM, sizeof(struct tm));
 
-    loadResult = configFileReader(filename, FALSE, loadPropertiesCallback, properties, TRUE, preload);
+    loadResult = configFileReader(filename, FALSE, loadPropertiesCallback, properties, TRUE, preload, argCommand, originalWorkingDir, properties->warnedVarMap, properties->logWarnings, properties->logWarningLogLevel, isDebugging);
 
     /* Any failure is a failure in the root. */
     switch (loadResult) {
@@ -528,19 +551,41 @@ int loadProperties(Properties *properties, const TCHAR* filename, int preload) {
     }
 }
 
-Properties* createProperties() {
+/**
+ * Get the log level of the messages reported when properties are overwritten.
+ *
+ * @param properties 
+ *
+ * @return log level 
+ */
+int GetLogLevelOnOverwrite(Properties *properties) {
+    /* Should be at least LEVEL_FATAL if exitOnOverwrite is set to TRUE */
+    if (properties) {
+        if (properties->exitOnOverwrite) {
+            return __max(properties->logLevelOnOverwrite, LEVEL_FATAL);
+        }
+        return properties->logLevelOnOverwrite;
+    }
+    return LEVEL_UNKNOWN;
+}
+
+Properties* createProperties(int debug, int logLevelOnOverwrite, int exitOnOverwrite) {
     Properties *properties = malloc(sizeof(Properties));
     if (!properties) {
         outOfMemory(TEXT("CP"), 1);
         return NULL;
     }
-    properties->debugProperties = FALSE;
+    properties->debugProperties = debug;
+    properties->exitOnOverwrite = exitOnOverwrite;
+    properties->logLevelOnOverwrite = logLevelOnOverwrite;
+    properties->overwrittenPropertyCausedExit = FALSE;
     properties->logWarnings = TRUE;
     properties->logWarningLogLevel = LEVEL_WARN;
     properties->first = NULL;
     properties->last = NULL;
     properties->warnedVarMap = newHashMap(8);
     if (!properties->warnedVarMap) {
+        disposeProperties(properties);
         return NULL;
     }
     return properties;
@@ -668,7 +713,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
 #ifdef WIN32
  #if defined(WRAPPER_USE_PUTENV_S)
             if (_tputenv_s(name, TEXT("")) == EINVAL) {
-                _tprintf(TEXT("Unable to clear environment variable: %s\n"), name);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
                 result = TRUE;
             }
  #else
@@ -681,7 +726,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s="), name);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to clear environment variable: %s\n"), name); 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
                     result = TRUE; 	 
                 }
             }
@@ -697,8 +742,8 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s="), name);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to clear environment variable: %s\n"), name); 	 
-                    result = TRUE; 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to clear the '%s' environment variable."), name);
+                    result = TRUE;
                 }
   #ifdef UNICODE
                 free(envBuf);
@@ -715,21 +760,26 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
 #ifdef WIN32
  #if defined(WRAPPER_USE_PUTENV_S)
             if (_tputenv_s(name, value) == EINVAL) {
-                _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set '%s% environment variable to: %s"), name, value);
                 result = TRUE;
             }
  #else
             len = _tcslen(name) + 1 + _tcslen(value) + 1;
-            envBuf = malloc(sizeof(TCHAR) * len);
-            if (!envBuf) {
-                outOfMemory(TEXT("SEI"), 2); 	 
+            if (len > MAX_ENV_PAIR_LEN) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable because total pair length of %d is longer than maximum for the OS of %d."), name, len,  MAX_ENV_PAIR_LEN);
                 result = TRUE;
             } else {
-                _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
-                /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
-                if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value); 	 
-                    result = TRUE; 	 
+                envBuf = malloc(sizeof(TCHAR) * len);
+                if (!envBuf) {
+                    outOfMemory(TEXT("SEI"), 2); 	 
+                    result = TRUE;
+                } else {
+                    _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
+                    /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
+                    if (_tputenv(envBuf)) {
+                        log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set environment variable: %s=%s"), name, value);
+                        result = TRUE; 	 
+                    }
                 }
             }
  #endif
@@ -744,8 +794,8 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
                 _sntprintf(envBuf, len, TEXT("%s=%s"), name, value);
                 /* The memory pointed to by envBuf should only be freed if this is UNICODE. */
                 if (_tputenv(envBuf)) { 	 
-                    _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value); 	 
-                    result = TRUE; 	 
+                    log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable to: %s"), name, value);
+                    result = TRUE;
                 }
   #ifdef UNICODE
                 free(envBuf);
@@ -753,7 +803,7 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
             }
  #else
             if (_tsetenv(name, value, TRUE)) {
-                _tprintf(TEXT("Unable to set environment variable: %s=%s\n"), name, value);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL, TEXT("Unable to set the '%s' environment variable to: %s"), name, value);
                 result = TRUE;
             }
  #endif
@@ -778,8 +828,11 @@ int setEnvInner(const TCHAR *name, const TCHAR *value) {
  *
  * @param name Name of the variable being set.
  * @param value Value to be set, NULL to clear it.
- * @param source Where the variable came from.  If value is WRAPPER_ENV_SOURCE_PARENT
- *               then the value may be NULL and will never be set to the environment.
+ * @param source Where the variable came from.
+ *               Must be one of ENV_SOURCE_PARENT, ENV_SOURCE_APPLICATION, ENV_SOURCE_CONFIG,
+ *                 or ENV_SOURCE_REG_SYSTEM or ENV_SOURCE_REG_ACCOUNT on Windows.
+ *               If value is ENV_SOURCE_PARENT then the value may be NULL and will never be
+ *                 set to the environment.
  *
  * Return TRUE if there were any problems, FALSE otherwise.
  */
@@ -856,37 +909,6 @@ int setEnv(const TCHAR *name, const TCHAR *value, int source) {
     return FALSE;
 }
 
-/* Trims any whitespace from the beginning and end of the in string
- *  and places the results in the out buffer.  Assumes that the out
- *  buffer is at least as large as the in buffer. */
-void trim(const TCHAR *in, TCHAR *out) {
-    size_t len;
-    size_t first;
-    size_t last;
-
-    len = _tcslen(in);
-    if (len > 0) {
-        first = 0;
-        last = len - 1;
-
-        /* Right Trim */
-        while (((in[first] == ' ') || (in[first] == '\t')) && (first < last)) {
-            first++;
-        }
-        /* Left Trim */
-        while ((last > first) && ((in[last] == ' ') || (in[last] == '\t'))) {
-            last--;
-        }
-
-        /* Copy over what is left. */
-        len = last - first + 1;
-        if (len > 0) {
-            _tcsncpy(out, in + first, len);
-        }
-    }
-    out[len] = TEXT('\0');
-}
-
 /**
  * Used to set a NULL terminated list of property names whose values should be
  *  escaped when read in from a file.   '\\' will become '\' and '\n' will
@@ -1100,6 +1122,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
     TCHAR *propertyNameTrim;
     TCHAR *propertyValueTrim;
     TCHAR *propertyExpandedValue;
+    int logLevelOnOverwrite;
 
 #ifdef _DEBUG
     log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT("addProperty(properties, %s, '%s', '%s', %d, %d, %d, %d)"),
@@ -1151,28 +1174,44 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
         /* Insert this property at the correct location.  Value will still be null. */
         insertInnerProperty(properties, property);
     } else {
-        /* The property was already set.  Only change it if non final */
-        if (property->internal) {
+        /* The property was already set.  Only change it if non final and non internal */
+        if (property->internal || property->finalValue) {
             setValue = FALSE;
-            
-            if (properties->debugProperties) {
-                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                    TEXT("The \"%s\" property is defined by the Wrapper internally and can not be overwritten.\n  Ignoring redefinition on line #%d of configuration file: %s\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
-                    propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+        }
+        
+        /* On preload we set properties->debugProperties to false as we don't want to log anything nor to stop the Wrapper at this stage. */  
+        if (properties->debugProperties) {
+            /* Preload was already done so the logging system is ready. */
+            logLevelOnOverwrite = GetLogLevelOnOverwrite(properties);
+            /* From version 3.5.27, the Wrapper will also log messages if the command line contains duplicated properties or attempts to set an internal environment variable. */
+            if (finalValue) {
+                if (property->internal) {
+                    log_printf(WRAPPER_SOURCE_WRAPPER, logLevelOnOverwrite,
+                        TEXT("The \"%s\" property is defined by the Wrapper internally and can not be overwritten.\n  Ignoring redefinition on the Wrapper command line.\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
+                        propertyNameTrim, propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+                } else if (property->finalValue) {
+                    log_printf(WRAPPER_SOURCE_WRAPPER, logLevelOnOverwrite,
+                        TEXT("The \"%s\" property was already defined on the Wrapper command line and can not be overwritten.\n  Ignoring redefinition on the Wrapper command line.\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
+                        propertyNameTrim, propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+                }
+            } else {
+                if (property->internal) {
+                    log_printf(WRAPPER_SOURCE_WRAPPER, logLevelOnOverwrite,
+                        TEXT("The \"%s\" property is defined by the Wrapper internally and can not be overwritten.\n  Ignoring redefinition on line #%d of configuration file: %s\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
+                        propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+                } else if (property->finalValue) {
+                    log_printf(WRAPPER_SOURCE_WRAPPER, logLevelOnOverwrite,
+                        TEXT("The \"%s\" property was defined on the Wrapper command line and can not be overwritten.\n  Ignoring redefinition on line #%d of configuration file: %s\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
+                        propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+                } else {
+                    log_printf(WRAPPER_SOURCE_WRAPPER, logLevelOnOverwrite,
+                        TEXT("The \"%s\" property was redefined on line #%d of configuration file: %s\n  Old Value %s=%s\n  New Value %s=%s"),
+                        propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+                }
             }
-        } else if (property->finalValue) {
-            setValue = FALSE;
             
-            if (properties->debugProperties) {
-                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                    TEXT("The \"%s\" property was defined on the Wrapper command line and can not be overwritten.\n  Ignoring redefinition on line #%d of configuration file: %s\n  Fixed Value %s=%s\n  Ignored Value %s=%s"),
-                    propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
-            }
-        } else {
-            if (properties->debugProperties) {
-                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                    TEXT("The \"%s\" property was redefined on line #%d of configuration file: %s\n  Old Value %s=%s\n  New Value %s=%s"),
-                    propertyNameTrim, lineNum, (filename ? filename : TEXT("<NULL>")), propertyNameTrim, property->value, propertyNameTrim, propertyValueTrim);
+            if (properties->exitOnOverwrite) {
+                properties->overwrittenPropertyCausedExit = TRUE;
             }
         }
     }
@@ -1204,6 +1243,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
         }
 
         if (property->value == NULL) {
+            free(propertyValueTrim);
             return NULL;
         }
         /* Store the final flag */
@@ -1230,7 +1270,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT("set default env('%s', '%s')"),
                     property->name + 12, property->value);
 #endif
-                setEnv(property->name + 12, property->value, (internal ? ENV_SOURCE_WRAPPER : ENV_SOURCE_CONFIG));
+                setEnv(property->name + 12, property->value, (internal ? ENV_SOURCE_APPLICATION : ENV_SOURCE_CONFIG));
             } else {
 #ifdef _DEBUG
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT(
@@ -1249,7 +1289,7 @@ Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS, TEXT("set env('%s', '%s')"),
                 property->name + 4, property->value);
 #endif
-            setEnv(property->name + 4, property->value, (internal ? ENV_SOURCE_WRAPPER : ENV_SOURCE_CONFIG));
+            setEnv(property->name + 4, property->value, (internal ? ENV_SOURCE_APPLICATION : ENV_SOURCE_CONFIG));
         }
     }
     free(propertyValueTrim);
@@ -1649,9 +1689,9 @@ int getBooleanProperty(Properties *properties, const TCHAR *propertyName, int de
     const TCHAR *propertyValue;
     
     if (defaultValue) {
-        defaultValueS = TEXT("true");
+        defaultValueS = TEXT("TRUE");
     } else {
-        defaultValueS = TEXT("false");
+        defaultValueS = TEXT("FALSE");
     }
 
     property = getInnerProperty(properties, propertyName, TRUE);
@@ -1661,21 +1701,94 @@ int getBooleanProperty(Properties *properties, const TCHAR *propertyName, int de
         propertyValue = property->value;
     }
     
-    if (strcmpIgnoreCase(propertyValue, TEXT("true")) == 0) {
+    if (strcmpIgnoreCase(propertyValue, TEXT("TRUE")) == 0) {
         return TRUE;
-    } else if (strcmpIgnoreCase(propertyValue, TEXT("false")) == 0) {
+    } else if (strcmpIgnoreCase(propertyValue, TEXT("FALSE")) == 0) {
         return FALSE;
     } else {
         if (properties->logWarnings) {
             log_printf(WRAPPER_SOURCE_WRAPPER, properties->logWarningLogLevel,
                 TEXT("Encountered an invalid boolean value for configuration property %s=%s.  Resolving to %s."),
-                propertyName, propertyValue, TEXT("FALSE"));
+                propertyName, propertyValue, defaultValueS);
         }
         
-        return FALSE;
+        return defaultValue;
     }
 }
 
+/**
+ * Build sorted arrays of all boolean properties beginning with {propertyNameBase}.
+ *  Only numerical characters can be returned between the two.
+ *
+ * The calling code must always call freeBooleanProperties to make sure that the
+ *  malloced propertyNames, propertyValues, and propertyIndices arrays are freed
+ *  up correctly.  This is only necessary if the function returns 0.
+ *
+ * @param properties The full properties structure.
+ * @param propertyNameHead All matching properties must begin with this value.
+ * @param propertyNameTail All matching properties must end with this value.
+ * @param all If FALSE then the array will start with #1 and loop up until the
+ *            next property is not found, if TRUE then all properties will be
+ *            returned, even if there are gaps in the series.
+ * @param matchAny If FALSE only numbers are allowed as placeholder
+ * @param propertyNames Returns a pointer to a NULL terminated array of
+ *                      property names.
+ * @param propertyValues Returns a pointer to a NULL terminated array of
+ *                       property values.
+ * @param propertyIndices Returns a pointer to a 0 terminated array of
+ *                        the index numbers used in each property name of
+ *                        the propertyNames array.
+ *
+ * @return 0 if successful, -1 if there was an error.
+ */
+int getBooleanProperties(Properties *properties, const TCHAR *propertyNameHead, const TCHAR *propertyNameTail, int all, int matchAny, TCHAR ***propertyNames, int **propertyValues, long unsigned int **propertyIndices, int defaultValue) {
+    TCHAR **strPropertyValues;
+    int i = 0;
+    int count = 0;
+    int result;
+    
+    result = getStringProperties(properties, propertyNameHead, propertyNameTail, all, matchAny, propertyNames, &strPropertyValues, propertyIndices);
+    if (result == -1)
+        return result;
+    
+    while (strPropertyValues[i]) {
+        count++;
+        i++;
+    }
+    *propertyValues = malloc(sizeof(TCHAR *) * (count + 1));
+    
+    i = 0;
+    while (strPropertyValues[i]) {
+        if (strcmpIgnoreCase(strPropertyValues[i], TEXT("TRUE")) == 0) {
+            (*propertyValues)[i] = TRUE;
+        } else if (strcmpIgnoreCase(strPropertyValues[i], TEXT("FALSE")) == 0) {
+            (*propertyValues)[i] = FALSE;
+        } else {
+            if (properties->logWarnings) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, properties->logWarningLogLevel,
+                    TEXT("Encountered an invalid boolean value for configuration property %s=%s.  Resolving to %s."),
+                    (*propertyNames)[i], strPropertyValues[i], defaultValue ? TEXT("TRUE") : TEXT("FALSE"));
+            }
+            
+            (*propertyValues)[i] = defaultValue;
+        }
+        i++;    
+    }
+    
+    free(strPropertyValues);
+    return 0;
+}
+
+void freeBooleanProperties(TCHAR **propertyNames, int *propertyValues, long unsigned int *propertyIndices) {
+    /* The property names are not malloced. */
+    free(propertyNames);
+
+    free(propertyValues);
+
+    free(propertyIndices);
+}
+
+
 int isQuotableProperty(Properties *properties, const TCHAR *propertyName) {
     Property *property;
     property = getInnerProperty(properties, propertyName, FALSE);
